@@ -1,39 +1,23 @@
 mod message;
+#[cfg(feature = "serde")]
+mod serde;
 
 use std::{
     borrow::Cow,
     collections::{BTreeMap, HashMap},
 };
 
-use prost::bytes::Bytes;
+#[cfg(feature = "serde")]
+pub use self::serde::{DeserializeOptions, SerializeOptions};
 
-use crate::{descriptor::Kind, FieldDescriptor, MessageDescriptor};
+use prost::{bytes::Bytes, DecodeError, Message};
 
-/// [`DynamicMessage`] allows reflection of a protobuf message.
+use crate::{descriptor::Kind, FieldDescriptor, MessageDescriptor, ReflectMessage};
+
+/// [`DynamicMessage`] provides encoding, decoding and reflection of a protobuf message.
 ///
 /// It wraps a [`MessageDescriptor`] and the [`Value`] for each field of the message, and implements
 /// [`Message`][`prost::Message`].
-///
-/// # Example - decoding
-///
-/// [`DynamicMessage`] does not implement [`Default`] since it needs a message descriptor to
-/// function. To decode a protobuf byte stream into an instance of this type, create a default
-/// value for the [`MessageDescriptor`] instance and merge into it:
-///
-/// ```
-/// use prost::Message;
-/// use prost_reflect::{DynamicMessage, FileDescriptor, Value};
-///
-/// # let bytes = b"\n\xaf\x01\n\x10my_message.proto\x12\x07package\"\x1d\n\tMyMessage\x12\x10\n\x03foo\x18\x01 \x01(\x05R\x03fooJk\n\x06\x12\x04\x00\x00\x06\x01\n\x08\n\x01\x0c\x12\x03\x00\x00\x12\n\x08\n\x01\x02\x12\x03\x02\x00\x10\n\n\n\x02\x04\x00\x12\x04\x04\x00\x06\x01\n\n\n\x03\x04\x00\x01\x12\x03\x04\x08\x11\n\x0b\n\x04\x04\x00\x02\x00\x12\x03\x05\x02\x10\n\x0c\n\x05\x04\x00\x02\x00\x05\x12\x03\x05\x02\x07\n\x0c\n\x05\x04\x00\x02\x00\x01\x12\x03\x05\x08\x0b\n\x0c\n\x05\x04\x00\x02\x00\x03\x12\x03\x05\x0e\x0fb\x06proto3";
-/// # let file_descriptor_set = prost_types::FileDescriptorSet::decode(bytes.as_ref()).unwrap();
-/// let file_descriptor = FileDescriptor::new(file_descriptor_set).unwrap();
-/// let message_descriptor = file_descriptor.get_message_by_name("package.MyMessage").unwrap();
-///
-/// let mut dynamic_message = DynamicMessage::new(message_descriptor);
-/// dynamic_message.merge(b"\x08\x96\x01".as_ref());
-///
-/// assert_eq!(dynamic_message.get_field_by_name("foo").unwrap().as_ref(), &Value::I32(150));
-/// ```
 #[derive(Debug, Clone, PartialEq)]
 pub struct DynamicMessage {
     desc: MessageDescriptor,
@@ -109,12 +93,6 @@ impl DynamicMessage {
         }
     }
 
-    /// Gets a reference to the [`MessageDescriptor`] containing protobuf type information for
-    /// this message.
-    pub fn descriptor(&self) -> MessageDescriptor {
-        self.desc.clone()
-    }
-
     /// Returns `true` if this message has a field set with the number `number`.
     ///
     /// If the field type supports distinguishing whether a value has been set, such as
@@ -206,6 +184,38 @@ impl DynamicMessage {
             self.clear_field(field_desc.number());
         }
     }
+
+    /// Merge a strongly-typed message into this one.
+    ///
+    /// The message should be compatible with the type specified by
+    /// [`descriptor`][Self::descriptor], or the merge will likely fail with
+    /// a [`DecodeError`].
+    pub fn transcode_from<T>(&mut self, value: &T) -> Result<(), DecodeError>
+    where
+        T: Message,
+    {
+        let buf = value.encode_to_vec();
+        self.merge(buf.as_slice())
+    }
+
+    /// Convert this dynamic message into a strongly typed value.
+    ///
+    /// The message should be compatible with the type specified by
+    /// [`descriptor`][Self::descriptor], or the merge will likely fail with
+    /// a [`DecodeError`].
+    pub fn transcode_to<T>(&self) -> Result<T, DecodeError>
+    where
+        T: Message + Default,
+    {
+        let buf = self.encode_to_vec();
+        T::decode(buf.as_slice())
+    }
+}
+
+impl ReflectMessage for DynamicMessage {
+    fn descriptor(&self) -> MessageDescriptor {
+        self.desc.clone()
+    }
 }
 
 impl DynamicMessageField {
@@ -453,6 +463,54 @@ impl Value {
             _ => None,
         }
     }
+
+    /// Returns a a reference to the value if it is a `Value::Message`, or `None` if it is any other type.
+    pub fn as_message(&self) -> Option<&DynamicMessage> {
+        match self {
+            Value::Message(value) => Some(value),
+            _ => None,
+        }
+    }
+
+    /// Returns a mutable reference to the value if it is a `Value::Message`, or `None` if it is any other type.
+    pub fn as_message_mut(&mut self) -> Option<&mut DynamicMessage> {
+        match self {
+            Value::Message(value) => Some(value),
+            _ => None,
+        }
+    }
+
+    /// Returns a a reference to the value if it is a `Value::List`, or `None` if it is any other type.
+    pub fn as_list(&self) -> Option<&[Value]> {
+        match self {
+            Value::List(value) => Some(value),
+            _ => None,
+        }
+    }
+
+    /// Returns a mutable reference to the value if it is a `Value::List`, or `None` if it is any other type.
+    pub fn as_list_mut(&mut self) -> Option<&mut Vec<Value>> {
+        match self {
+            Value::List(value) => Some(value),
+            _ => None,
+        }
+    }
+
+    /// Returns a a reference to the value if it is a `Value::Map`, or `None` if it is any other type.
+    pub fn as_map(&self) -> Option<&HashMap<MapKey, Value>> {
+        match self {
+            Value::Map(value) => Some(value),
+            _ => None,
+        }
+    }
+
+    /// Returns a mutable reference to the value if it is a `Value::Map`, or `None` if it is any other type.
+    pub fn as_map_mut(&mut self) -> Option<&mut HashMap<MapKey, Value>> {
+        match self {
+            Value::Map(value) => Some(value),
+            _ => None,
+        }
+    }
 }
 
 impl MapKey {
