@@ -23,7 +23,7 @@
 //! let message_descriptor = file_descriptor.get_message_by_name("package.MyMessage").unwrap();
 //!
 //! let mut dynamic_message = DynamicMessage::new(message_descriptor);
-//! dynamic_message.merge(b"\x08\x96\x01".as_ref());
+//! dynamic_message.merge(b"\x08\x96\x01".as_ref()).unwrap();
 //!
 //! assert_eq!(dynamic_message.get_field_by_name("foo").unwrap().as_ref(), &Value::I32(150));
 //! ```
@@ -85,19 +85,21 @@ assert_eq!(message.descriptor().full_name(), "package.MyMessage");
 ```
 
 If you are using `prost-build`, it can be configured to generate [`ReflectMessage`] implementations
-for a package:
+for messages:
 
 ```rust,no_run
 use prost_build::Config;
 
 Config::new()
     .file_descriptor_set_path("file_descriptor_set.bin")
-    .type_attribute("package", "#[prost_reflect(file_descriptor = \"FILE_DESCRIPTOR\", message_name = \"package.MyMessage\")]")
+    .type_attribute(".package.MyMessage", "#[derive(::prost_reflect::ReflectMessage)]")
+    .type_attribute(".package.MyMessage", "#[prost_reflect(file_descriptor = \"FILE_DESCRIPTOR\", message_name = \"package.MyMessage\")]")
     .compile_protos(&["src/package.proto"], &["src"])
     .unwrap();
 ```
 "##
 )]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 #![warn(missing_debug_implementations, missing_docs)]
 
 #[cfg(feature = "serde1")]
@@ -118,4 +120,5 @@ pub use self::reflect::ReflectMessage;
 pub use self::dynamic::{DeserializeOptions, SerializeOptions};
 
 #[cfg(feature = "derive")]
+#[cfg_attr(docsrs, doc(cfg(feature = "derive")))]
 pub use prost_reflect_derive::ReflectMessage;
